# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import os
import re
import runpy
import subprocess

from setuptools import setup, find_packages
from io import open

# Version information; update this by hand when making a new bugfix or feature
# release. The actual package version is autogenerated from this information
# together with information from the version control system, and then injected
# into the package source.
MAJOR = 7
MINOR = 2
MICRO = 1
PRERELEASE = ""
IS_RELEASED = True

# If this file is part of a Git export (for example created with "git archive",
# or downloaded from GitHub), ARCHIVE_COMMIT_HASH gives the full hash of the
# commit that was exported.
ARCHIVE_COMMIT_HASH = "$Format:%H$"

# Templates for version strings.
RELEASED_VERSION = "{major}.{minor}.{micro}{prerelease}"
UNRELEASED_VERSION = "{major}.{minor}.{micro}{prerelease}.dev{dev}"

# Paths to the autogenerated version file and the Git directory.
HERE = os.path.abspath(os.path.dirname(__file__))
VERSION_FILE = os.path.join(HERE, "traitsui", "_version.py")
GIT_DIRECTORY = os.path.join(HERE, ".git")

# Template for the autogenerated version file.
VERSION_FILE_TEMPLATE = """\
# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

# THIS FILE IS GENERATED FROM SETUP.PY

#: The full version of the package, including a development suffix
#: for unreleased versions of the package.
version = '{version}'

#: The full version of the package, same as 'version'
#: Kept for backward compatibility
full_version = version

#: The Git revision from which this release was made.
git_revision = '{git_revision}'

#: Flag whether this is a final release
is_released = {is_released}
"""


def read_module(module, package='traitsui'):
    """ Read a simple .py file from traitsui in a safe way.

    It would be simpler to import the file, but that can be problematic in an
    unknown system, so we exec the file instead and extract the variables.

    This will fail if things get too complex in the file being read, but is
    sufficient to get version and requirements information.
    """
    base_dir = os.path.dirname(__file__)
    module_name = package + '.' + module
    path = os.path.join(base_dir, package, module+'.py')
    with open(path, 'r', encoding='utf-8') as fp:
        code = compile(fp.read(), module_name, 'exec')
    context = {}
    exec(code, context)
    return context


# Return the git revision as a string
def _git_info():
    """
    Get information about the given commit from Git.

    Returns
    -------
    git_count : int
        Number of revisions from this commit to the initial commit.
    git_revision : str
        Commit hash for HEAD.

    Raises
    ------
    EnvironmentError
        If Git is not available.
    subprocess.CalledProcessError
        If Git is available, but the version command fails (most likely
        because there's no Git repository here).
    """
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(
            cmd, stdout=subprocess.PIPE, env=env,
        ).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'describe', '--tags'])
    except OSError:
        out = ''

    git_description = out.strip().decode('ascii')
    expr = r'.*?\-(?P<count>\d+)-g(?P<hash>[a-fA-F0-9]+)'
    match = re.match(expr, git_description)
    if match is None:
        git_revision, git_count = 'Unknown', '0'
    else:
        git_revision, git_count = match.group('hash'), match.group('count')

    return git_count, git_revision


def git_version():
    """
    Construct version information from local variables and Git.

    Returns
    -------
    version : str
        Package version.
    git_revision : str
        The full commit hash for the current Git revision.

    Raises
    ------
    EnvironmentError
        If Git is not available.
    subprocess.CalledProcessError
        If Git is available, but the version command fails (most likely
        because there's no Git repository here).
    """
    git_count, git_revision = _git_info()
    version_template = RELEASED_VERSION if IS_RELEASED else UNRELEASED_VERSION
    version = version_template.format(
        major=MAJOR,
        minor=MINOR,
        micro=MICRO,
        prerelease=PRERELEASE,
        dev=git_count,
    )
    return version, git_revision


def archive_version():
    """
    Construct version information for an archive.

    Returns
    -------
    version : str
        Package version.
    git_revision : str
        The full commit hash for the current Git revision.

    Raises
    ------
    ValueError
        If this does not appear to be an archive.
    """
    if "$" in ARCHIVE_COMMIT_HASH:
        raise ValueError("This does not appear to be an archive.")

    version_template = RELEASED_VERSION if IS_RELEASED else UNRELEASED_VERSION
    version = version_template.format(
        major=MAJOR,
        minor=MINOR,
        micro=MICRO,
        prerelease=PRERELEASE,
        dev="-unknown",
    )
    return version, ARCHIVE_COMMIT_HASH


def write_version_file(version, git_revision, filename=VERSION_FILE):
    """
    Write version information to the version file.

    Overwrites any existing version file.

    Parameters
    ----------
    version : str
        Package version.
    git_revision : str
        The full commit hash for the current Git revision.
    filename : str
        Path to the version file.
    """
    with open(filename, "w", encoding="utf-8") as version_file:
        version_file.write(
            VERSION_FILE_TEMPLATE.format(
                version=version,
                git_revision=git_revision,
                is_released=IS_RELEASED,
            )
        )


def read_version_file():
    """
    Read version information from the version file, if it exists.

    Returns
    -------
    version : str
        The full version, including any development suffix.
    git_revision : str
        The full commit hash for the current Git revision.

    Raises
    ------
    EnvironmentError
        If the version file does not exist.
    """
    version_info = runpy.run_path(VERSION_FILE)
    return (version_info["version"], version_info["git_revision"])


def resolve_version():
    """
    Process version information and write a version file if necessary.

    Returns the current version information.

    Returns
    -------
    version : str
        Package version.
    git_revision : str
        The full commit hash for the current Git revision.
    """
    if os.path.isdir(GIT_DIRECTORY):
        # This is a local clone; compute version information and write
        # it to the version file, overwriting any existing information.
        version = git_version()
        print("Computed package version: {}".format(version))
        print("Writing version to version file {}.".format(VERSION_FILE))
        write_version_file(*version)
    elif "$" not in ARCHIVE_COMMIT_HASH:
        # This is a source archive.
        version = archive_version()
        print("Archive package version: {}".format(version))
        print("Writing version to version file {}.".format(VERSION_FILE))
        write_version_file(*version)
    elif os.path.isfile(VERSION_FILE):
        # This is a source distribution. Read the version information.
        print("Reading version file {}".format(VERSION_FILE))
        version = read_version_file()
        print("Package version from version file: {}".format(version))
    else:
        raise RuntimeError(
            "Unable to determine package version. No local Git clone "
            "detected, and no version file found at {}.".format(VERSION_FILE)
        )

    return version


if __name__ == "__main__":
    __version__, _ = resolve_version()
    data = read_module('__init__')
    __requires__ = data['__requires__']
    __extras_require__ = data['__extras_require__']
    with open("README.rst", "r", encoding="utf-8") as readme:
        LONG_DESCRIPTION = readme.read().split(".. end_of_long_description")[0]

    def additional_commands():
        try:
            from sphinx.setup_command import BuildDoc
        except ImportError:
            return {}
        else:
            return {'documentation': BuildDoc}

    setup(
        name='traitsui',
        version=__version__,
        author='David C. Morrill, et. al.',
        author_email='dmorrill@enthought.com',
        classifiers=[c.strip() for c in """\
            Development Status :: 5 - Production/Stable
            Intended Audience :: Developers
            Intended Audience :: Science/Research
            License :: OSI Approved :: BSD License
            Operating System :: MacOS
            Operating System :: Microsoft :: Windows
            Operating System :: OS Independent
            Operating System :: POSIX
            Operating System :: Unix
            Programming Language :: Python
            Programming Language :: Python :: 3
            Programming Language :: Python :: 3.6
            Programming Language :: Python :: 3.7
            Programming Language :: Python :: 3.8
            Topic :: Scientific/Engineering
            Topic :: Software Development
            Topic :: Software Development :: Libraries
            """.splitlines() if len(c.strip()) > 0],
        description='traitsui: traits-capable user interfaces',
        long_description=LONG_DESCRIPTION,
        long_description_content_type="text/x-rst",
        url='http://docs.enthought.com/traitsui',
        download_url='https://github.com/enthought/traitsui',
        install_requires=__requires__,
        extras_require=__extras_require__,
        license='BSD',
        maintainer='ETS Developers',
        maintainer_email='enthought-dev@enthought.com',
        package_data=dict(
            traitsui=[
                'examples/demo/*',
                'examples/demo/*/*',
                'examples/demo/*/*/*',
                'extras/images/*',
                'image/library/*.zip',
                'images/*',
                'wx/images/*',
                'qt4/images/*',
                'testing/data/*',
            ]),
        packages=find_packages(),
        entry_points={
            'traitsui.toolkits': [
                'qt4 = traitsui.qt4:toolkit',
                'wx = traitsui.wx:toolkit',
                'qt = traitsui.qt4:toolkit',
                'null = traitsui.null:toolkit',
            ],
            'etsdemo_data': [
                'demo = traitsui.extras._demo_info:info',
            ]
        },
        platforms=["Windows", "Linux", "Mac OS-X", "Unix", "Solaris"],
        python_requires=">=3.6",
        zip_safe=False,
    )



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

# array_editor.py -- Example of using array editors

import numpy as np

from traits.api import Array, HasPrivateTraits
from traitsui.api import ArrayEditor, Item, View
from traitsui.menu import NoButtons


class ArrayEditorTest(HasPrivateTraits):

    three = Array(np.int, (3, 3))

    four = Array(
        np.float, (4, 4), editor=ArrayEditor(width=-50),
    )

    view = View(
        Item('three', label='3x3 Integer'),
        '_',
        Item('three', label='Integer Read-only', style='readonly'),
        '_',
        Item('four', label='4x4 Float'),
        '_',
        Item('four', label='Float Read-only', style='readonly'),
        buttons=NoButtons,
        resizable=True,
    )


if __name__ == '__main__':
    ArrayEditorTest().configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines a text editor which displays a text field and maintains a history
    of previously entered values.
"""


from traits.api import Any, observe

from pyface.timer.api import do_later

from .editor import Editor

from .history_control import HistoryControl

# -------------------------------------------------------------------------
#  '_HistoryEditor' class:
# -------------------------------------------------------------------------


class _HistoryEditor(Editor):
    """ Simple style text editor, which displays a text field and maintains a
        history of previously entered values, the maximum number of which is
        specified by the 'entries' trait of the HistoryEditor factory.
    """

    # -------------------------------------------------------------------------
    #  Trait definitions:
    # -------------------------------------------------------------------------

    #: The history control:
    history = Any()

    def init(self, parent):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self.history = history = HistoryControl(
            value=self.value,
            entries=self.factory.entries,
            auto_set=self.factory.auto_set,
        )
        self.control = history.create_control(parent)

        self.set_tooltip()

    def dispose(self):
        """ Disposes of the contents of an editor.
        """
        self.history.dispose()
        self.history = None

        super().dispose()

    @observe("history:value")
    def _value_changed(self, event):
        """ Handles the history object's 'value' trait being changed.
        """
        if not self._dont_update:
            history = self.history
            try:
                self._dont_update = True
                self.value = history.value
                history.error = False
            except:
                history.error = True

            do_later(self.trait_set, _dont_update=False)

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        if not self._dont_update:
            self._dont_update = True
            self.history.value = self.value
            self.history.error = False
            self._dont_update = False

    def error(self, excp):
        """ Handles an error that occurs while setting the object's trait value.
        """
        pass

    # -- UI preference save/restore interface ---------------------------------

    def restore_prefs(self, prefs):
        """ Restores any saved user preference information associated with the
            editor.
        """
        self.history.history = prefs.get("history", [])[: self.factory.entries]

    def save_prefs(self):
        """ Returns any user preference information associated with the editor.
        """
        # If the view closed successfully, try to update the history with the
        # current value:
        if self.ui.result:
            self._dont_update = True
            self.history.set_value(self.value)
            self._dont_update = False

        return {"history": self.history.history[:]}



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import unittest

from traits.api import HasStrictTraits
from traitsui import ui_traits
from traitsui.tests._tools import BaseTestMixin


class ObjectWithUITraits(HasStrictTraits):

    orientation = ui_traits.Orientation
    style = ui_traits.EditorStyle
    layout = ui_traits.Layout
    an_object = ui_traits.AnObject
    dock_style = ui_traits.DockStyle
    view_status = ui_traits.ViewStatus()


class TestUITraits(BaseTestMixin, unittest.TestCase):

    def setUp(self):
        BaseTestMixin.setUp(self)

    def tearDown(self):
        BaseTestMixin.tearDown(self)

    def test_orientation(self):
        obj = ObjectWithUITraits()
        self.assertEqual(obj.orientation, "vertical")

        obj.orientation = "h"
        self.assertEqual(obj.orientation, "horizontal")

    def test_editor_style(self):
        obj = ObjectWithUITraits()
        self.assertEqual(obj.style, "simple")

        obj.style = "r"
        self.assertEqual(obj.style, "readonly")

    def test_layout(self):
        obj = ObjectWithUITraits()
        self.assertEqual(obj.layout, "normal")

    def test_an_object(self):
        obj = ObjectWithUITraits()
        obj.an_object = "[1,2,3][0]"
        self.assertEqual(obj.an_object, "[1,2,3][0]")
        actual = eval(obj.an_object_, {}, {})
        self.assertEqual(actual, 1)


class TestStatusItem(BaseTestMixin, unittest.TestCase):

    def setUp(self):
        BaseTestMixin.setUp(self)

    def tearDown(self):
        BaseTestMixin.tearDown(self)

    def test_init(self):
        item = ui_traits.StatusItem(
            name="some_trait_name",
            width=10.0,
        )
        self.assertEqual(item.name, "some_trait_name")

    def test_init_with_name_and_value(self):
        # value trumps name. Not sure why but this is the way it is.
        item = ui_traits.StatusItem(
            name="some_trait_name",
            width=10.0,
            value="some_other_name",
        )
        self.assertEqual(item.name, "some_other_name")


class TestViewStatus(BaseTestMixin, unittest.TestCase):

    def setUp(self):
        BaseTestMixin.setUp(self)

    def tearDown(self):
        BaseTestMixin.tearDown(self)

    def test_init(self):
        obj = ObjectWithUITraits()

        obj.view_status = "some_name"
        self.assertEqual(len(obj.view_status), 1)
        status, = obj.view_status
        self.assertIsInstance(status, ui_traits.StatusItem)
        self.assertEqual(status.name, "some_name")



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

import unittest

from traits.api import Enum, HasTraits, Int, Str, Instance
from traitsui.api import HGroup, Item, Group, VGroup, View
from traitsui.menu import ToolBar, Action
from traitsui.testing.api import Index, IsVisible, MouseClick, UITester
from traitsui.tests._tools import (
    create_ui,
    requires_toolkit,
    process_cascade_events,
    ToolkitName,
)


class FooPanel(HasTraits):

    my_int = Int(2)
    my_str = Str("I am a panel/subpanel")
    toolbar = Instance(ToolBar)

    def default_traits_view(self):
        view = View(
            Item(name="my_int"),
            Item(name="my_str"),
            title="FooPanel",
            buttons=["OK", "Cancel"],
            toolbar=self.toolbar,
        )
        return view

    def _toolbar_default(self):
        return ToolBar(Action(name="Open file"))


class FooDialog(HasTraits):

    panel1 = Instance(FooPanel)
    panel2 = Instance(FooPanel)

    view = View(
        Group(Item("panel1"), Item("panel2"), layout="split", style="custom")
    )

    def _panel1_default(self):
        return FooPanel()

    def _panel2_default(self):
        return FooPanel()


class ScrollableGroupExample(HasTraits):

    my_int = Int(2)

    my_str = Str("The group is scrollable")


scrollable_group_view = View(
    Group(
        Item(name="my_int"),
        Item(name="my_str"),
        scrollable=True,
    ),
    title="FooPanel",
    kind='subpanel',
)

non_scrollable_group_view = View(
    Group(
        Item(name="my_int"),
        Item(name="my_str"),
        scrollable=False,
    ),
    title="FooPanel",
    kind='subpanel',
)

scrollable_group_box_view = View(
    Group(
        Item(name="my_int"),
        Item(name="my_str"),
        scrollable=True,
        label="Scrollable View",
        show_border=True,
    ),
    title="FooPanel",
    kind='subpanel',
)

scrollable_labelled_group_view = View(
    Group(
        Item(name="my_int"),
        Item(name="my_str"),
        scrollable=True,
        label="Scrollable View",
    ),
    title="FooPanel",
    kind='subpanel',
)


class ScrollableGroupVisibleWhen(HasTraits):
    bar = Str("bar!")
    baz = Str("Baz?")
    enabled = Enum("Yes", "No")

    def default_traits_view(self):
        view = View(
            Item("enabled"),
            HGroup(
                VGroup(
                    Item("bar"),
                    scrollable=True,
                    visible_when="enabled=='Yes'",
                    id='bar_group'
                ),
                VGroup(
                    Item("baz"),
                    scrollable=True,
                    visible_when="enabled=='No'",
                    id='baz_group'
                ),
            ),
        )
        return view


@requires_toolkit([ToolkitName.qt])
class TestUIPanel(unittest.TestCase):
    def setup_qt4_dock_window(self):
        from pyface.qt import QtGui

        # set up the dock window for qt
        main_window = QtGui.QMainWindow()
        self.addCleanup(process_cascade_events)
        self.addCleanup(main_window.close)
        dock = QtGui.QDockWidget("testing", main_window)
        dock.setWidget(QtGui.QMainWindow())
        return main_window, dock

    def test_panel_has_toolbar_buttons_qt4(self):
        from pyface.qt import QtGui

        _, dock = self.setup_qt4_dock_window()

        # add panel
        panel = FooPanel()
        with create_ui(panel, dict(parent=dock.widget(), kind="panel")) as ui:
            dock.widget().setCentralWidget(ui.control)

            # There should be a toolbar for the panel
            self.assertIsNotNone(dock.findChild(QtGui.QToolBar))

            # There should be buttons too
            # Not searching from dock because the dock panel has buttons for
            # popping up and closing the panel
            self.assertIsNotNone(ui.control.findChild(QtGui.QPushButton))

    def test_subpanel_has_toolbar_no_buttons_qt4(self):
        from pyface.qt import QtGui

        _, dock = self.setup_qt4_dock_window()

        # add panel
        panel = FooPanel()
        parent = dock.widget()
        with create_ui(panel, dict(parent=parent, kind="subpanel")) as ui:
            dock.widget().setCentralWidget(ui.control)

            # There should be a toolbar for the subpanel
            self.assertIsNotNone(dock.findChild(QtGui.QToolBar))

            # Buttons should not be shown for subpanel
            # Not searching from dock because the dock panel has buttons for
            # popping up and closing the panel
            self.assertIsNone(ui.control.findChild(QtGui.QPushButton))

    def test_subpanel_no_toolbar_nor_button_in_widget(self):
        from pyface.qt import QtGui

        # FooDialog uses a QWidget to contain the panels
        # No attempt should be made for adding the toolbars
        foo_window = FooDialog()
        with create_ui(foo_window) as ui:
            # No toolbar for the dialog
            self.assertIsNone(ui.control.findChild(QtGui.QToolBar))

            # No button
            self.assertIsNone(ui.control.findChild(QtGui.QPushButton))

    # regression test for enthought/traitsui#1512
    def test_scrollable_group_visible_when(self):
        from pyface.qt import QtGui

        obj = ScrollableGroupVisibleWhen()
        tester = UITester()
        with tester.create_ui(obj) as ui:
            bar_group = tester.find_by_id(ui, 'bar_group')
            baz_group = tester.find_by_id(ui, 'baz_group')

            # for a scrollable group the GroupEditors control should be a
            # QScrollArea not just the QWidget.  We want the full area to be
            # not visible, not just the text box widget.
            self.assertIsInstance(bar_group._target.control, QtGui.QScrollArea)
            self.assertIsInstance(baz_group._target.control, QtGui.QScrollArea)

            self.assertTrue(bar_group.inspect(IsVisible()))
            self.assertFalse(baz_group.inspect(IsVisible()))

            enabled_box = tester.find_by_name(ui, 'enabled')
            baz_item = enabled_box.locate(Index(1))
            baz_item.perform(MouseClick())

            self.assertTrue(baz_group.inspect(IsVisible()))
            self.assertFalse(bar_group.inspect(IsVisible()))


@requires_toolkit([ToolkitName.qt])
class TestPanelLayout(unittest.TestCase):

    def test_scrollable_group_typical(self):
        from pyface.qt import QtGui

        example = ScrollableGroupExample()

        ui = example.edit_traits(view=scrollable_group_view)
        try:
            mainwindow = ui.control.layout().itemAt(0).widget()
            scroll_area = mainwindow.centralWidget()
            self.assertIsInstance(scroll_area, QtGui.QScrollArea)
            content = scroll_area.widget()
            self.assertEqual(type(content), QtGui.QWidget)
        finally:
            ui.dispose()

    def test_scrollable_group_box(self):
        from pyface.qt import QtGui

        example = ScrollableGroupExample()

        ui = example.edit_traits(view=scrollable_group_box_view)
        try:
            mainwindow = ui.control.layout().itemAt(0).widget()
            scroll_area = mainwindow.centralWidget()
            self.assertIsInstance(scroll_area, QtGui.QScrollArea)
            group_box = scroll_area.widget()
            self.assertIsInstance(group_box, QtGui.QGroupBox)
            self.assertEqual(group_box.title(), "Scrollable View")
        finally:
            ui.dispose()

    def test_scrollable_labelled_group(self):
        from pyface.qt import QtGui

        example = ScrollableGroupExample()

        ui = example.edit_traits(view=scrollable_labelled_group_view)
        try:
            mainwindow = ui.control.layout().itemAt(0).widget()
            scroll_area = mainwindow.centralWidget()
            self.assertIsInstance(scroll_area, QtGui.QScrollArea)
            content = scroll_area.widget()
            self.assertEqual(type(content), QtGui.QWidget)
        finally:
            ui.dispose()

    def test_non_scrollable_group_typical(self):
        from pyface.qt import QtGui

        example = ScrollableGroupExample(my_str="The group is not scrollable")

        ui = example.edit_traits(view=non_scrollable_group_view)
        try:
            mainwindow = ui.control.layout().itemAt(0).widget()
            content = mainwindow.centralWidget()
            self.assertEqual(type(content), QtGui.QWidget)
        finally:
            ui.dispose()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines array editors for the WX user interface toolkit.
"""


from traitsui.editors.array_editor import SimpleEditor as BaseSimpleEditor

from .editor import Editor

# -------------------------------------------------------------------------
#  'SimpleEditor' class:
# -------------------------------------------------------------------------


class SimpleEditor(BaseSimpleEditor, Editor):
    """ Simple style of editor for arrays.
    """

    # FIXME: This class has been re-defined here simply so it inherits from the
    # wx Editor class.
    pass


class ReadonlyEditor(SimpleEditor):

    #: Set the value of the readonly trait.
    readonly = True



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines the BasicUIEditor class, which allows creating editors that define
    their function by creating an embedded Traits UI.
"""


from traitsui.ui_editor import UIEditor as BaseUIEditor

from .editor import Editor

# -------------------------------------------------------------------------
#  'UIEditor' base class:
# -------------------------------------------------------------------------


class UIEditor(BaseUIEditor, Editor):
    """ An editor that creates an embedded Traits UI.
    """

    pass



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Tests pertaining to the AnimatedGIFEditor
"""

import unittest

import pkg_resources

from traits.api import File, HasTraits
from traitsui.api import Item, View
from traitsui.tests._tools import (
    BaseTestMixin,
    create_ui,
    requires_toolkit,
    ToolkitName,
)

filename = pkg_resources.resource_filename(
    "traitsui", "examples/demo/Extras/images/logo_32x32.gif"
)


class AnimatedGIF(HasTraits):

    gif_file = File(filename)


@requires_toolkit([ToolkitName.wx])
class TestAnimatedGIFEditor(BaseTestMixin, unittest.TestCase):

    def setUp(self):
        BaseTestMixin.setUp(self)

    def tearDown(self):
        BaseTestMixin.tearDown(self)

    def test_animated_gif_editor(self):
        from traitsui.wx.animated_gif_editor import AnimatedGIFEditor
        # Regression test for enthought/traitsui#1071
        obj1 = AnimatedGIF()
        view = View(
            Item('gif_file', editor=AnimatedGIFEditor(playing='playing')),
        )

        # This should not fail.
        with create_ui(obj1, dict(view=view)):
            pass



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
**WARNING**

  This demo might not work as expected and some documented features might be
  missing.

-------------------------------------------------------------------------------

Implementation of a DateEditor demo plugin for Traits UI demo program.

This demo shows a few different styles of the DateEditor and how it can be
customized.
"""
# Issue related to the demo warning: enthought/traitsui#962

from traits.api import HasTraits, Date, List, Str
from traitsui.api import View, Item, DateEditor, Group


class DateEditorDemo(HasTraits):
    """ Demo class to show Date editors. """
    single_date = Date()
    multi_date = List(Date)
    info_string = Str('The editors for Traits Date objects.  Showing both '
                      'the defaults, and one with alternate options.')

    multi_select_editor = DateEditor(
        allow_future=False,
        multi_select=True,
        shift_to_select=False,
        on_mixed_select='max_change',
        # Qt ignores these setting and always shows only 1 month:
        months=2,
        padding=30,
    )

    traits_view = View(
        Item('info_string', show_label=False, style='readonly'),
        Group(
            Item('single_date', label='Simple date editor'),
            Item('single_date', style='custom', label='Default custom editor'),
            Item(
                'single_date',
                style='readonly',
                editor=DateEditor(
                    strftime='You picked %B %d %Y',
                    message='Click a date above.'
                ),
                label='ReadOnly editor'
            ),
            label='Default settings for editors'
        ),
        Group(
            Item(
                'multi_date',
                editor=multi_select_editor,
                style='custom',
                label='Multi-select custom editor'
            ),
            label='More customized editor: multi-select; disallow '
                  'future; selection style; etc.'
        ),
        resizable=True
    )

    def _multi_date_changed(self):
        """ Print each time the date value is changed in the editor. """
        print(self.multi_date)

    def _simple_date_changed(self):
        """ Print each time the date value is changed in the editor. """
        print(self.simple_date, self.single_date)

    def _single_date_changed(self):
        """ Print each time the date value is changed in the editor. """
        print(self.single_date)


# -- Set Up The Demo ------------------------------------------------------

demo = DateEditorDemo()

if __name__ == "__main__":
    demo.configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

# DataFrameEditor_demo.py -- Example of using dataframe editors

# Dataset from https://www.kaggle.com/mokosan/lord-of-the-rings-character-data
"""
**WARNING**

  This demo might not work as expected and some documented features might be
  missing.

-------------------------------------------------------------------------------

To run this demonstration successfully, you must have **pandas** installed.

Please refer to the `DataFrameEditor API docs`_ for further information.

.. _DataFrameEditor API docs: https://docs.enthought.com/traitsui/api/traitsui.ui_editors.data_frame_editor.html#traitsui.ui_editors.data_frame_editor.DataFrameEditor
"""
# Issue related to the demo warning: enthought/traitsui#944


import numpy as np
from pandas import DataFrame

from traits.api import HasTraits, Instance

from traitsui.api import View, Item
from traitsui.ui_editors.data_frame_editor import DataFrameEditor


class DataFrameEditorDemo(HasTraits):

    df = Instance(DataFrame)

    traits_view = View(
        Item(
            'df',
            show_label=False,
            editor=DataFrameEditor(
                formats={
                    'RuntimeInMinutes': '%.4d',
                    'BudgetInMillions': '%d',
                    'BoxOfficeRevenueInMillions': '%d',
                    'AcademyAwardNominations': '%d',
                    'AcademyAwardWins': '%d',
                    'RottenTomatoesScore': '%.2f'
                }
            )
        ),
        title="DataFrameEditor",
        resizable=True,
        id='traitsui.demo.Applications.data_frame_editor_demo'
    )


# Sample Data
lotrMovieData = np.array(
    [
        [558, 281, 2917.0, 30, 17, 94.0],
        [178, 93, 871.5, 13, 4, 91.0],
        [179, 94, 926.0, 6, 2, 96.0],
        [201, 94, 1120, 11, 11, 95.0],
        [462.0, 675, 2932.0, 7, 1, 66.33333333],
        [169, 200, 1021.0, 3, 1, 64.0],
        [161, 217, 958.4, 3, 0, 75.0],
        [144, 250, 956.0, 1, 0, 60.0]
    ]
)

col_names = [
    'RuntimeInMinutes',
    'BudgetInMillions',
    'BoxOfficeRevenueInMillions',
    'AcademyAwardNominations',
    'AcademyAwardWins',
    'RottenTomatoesScore'
            ]

index_names = [
    'The Lord of the Rings Series',
    'The Fellowship of the Ring',
    'The Two Towers ',
    'The Return of the King',
    'The Hobbit Series',
    'The Unexpected Journey',
    'The Desolation of Smaug',
    'The Battle of the Five Armies'
]

# Create & run the demo
df = DataFrame(data=lotrMovieData, columns=col_names, index=index_names)

demo = DataFrameEditorDemo(df=df)

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import Bool, List, Str

from traitsui.editors.date_editor import DateEditor


class StyledDateEditor(DateEditor):
    """ A DateEditor that can show sets of dates in different styles.
    """

    #: The name of a dictionary on the object that maps names to groups
    #: (list/tuples) of datetime.date objects.  Each of these groups can be
    #: styled using the **styles** dict.
    dates_trait = Str()

    #: The name of a dictionary on the object that maps names of styles to
    #: CellFormat objects.  The names used must match the names used in the
    #: **dates** dict.
    styles_trait = Str()

    #: Allow selection of arbitrary dates in the past.
    allow_past = Bool(True)

    #: Allow selection of arbitrary dates in the future.
    allow_future = Bool(True)

    #: A list of strings that will be offered as an alternative to specifying
    #: an absolute date, and instead specify a relative date.
    relative_dates = List()


# This alias is deprecated and will be removed in TraitsUI 8.
ToolkitEditorFactory = StyledDateEditor



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" This module defines objects for locating nested UI targets, to be
used with ``UIWrapper.locate``.

Implementations for these actions are expected to return a value which is
a UI target where further location resolution or user interaction can be
applied.
"""


class Index:
    """ A locator for locating a target that is uniquely specified by a single
    0-based index.

    Attributes
    ----------
    index : int
        0-based index
    """

    def __init__(self, index):
        self.index = index


class TargetByName:
    """ A locator for locating the next UI target using a name.

    Attributes
    ----------
    name : str
    """
    def __init__(self, name):
        self.name = name


class TargetById:
    """ A locator for locating the next UI target using an id.

    Attributes
    ----------
    id : str
    """
    def __init__(self, id):
        self.id = id


class Slider:
    """ A locator for locating a nested slider widget within a UI.
    """
    pass


class Textbox:
    """ A locator for locating a nested textbox widget within a UI.
    """
    pass



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
Test cases for the UI object.
"""

import contextlib
import unittest

from pyface.api import GUI
from traits.api import Property
from traits.has_traits import HasTraits, HasStrictTraits
from traits.trait_types import Str, Int

import traitsui
from traitsui.basic_editor_factory import BasicEditorFactory
from traitsui.api import Group, Item, spring, View
from traitsui.testing.api import DisplayedText, IsEnabled, MouseClick, UITester
from traitsui.tests._tools import (
    BaseTestMixin,
    count_calls,
    create_ui,
    is_qt,
    is_wx,
    process_cascade_events,
    requires_toolkit,
    reraise_exceptions,
    ToolkitName,
)
from traitsui.toolkit import toolkit, toolkit_object


class FooDialog(HasTraits):
    """Test dialog that does nothing useful."""

    my_int = Int(2)
    my_str = Str("hallo")

    traits_view = View(Item("my_int"), Item("my_str"), buttons=["OK"])


class DisallowNewTraits(HasStrictTraits):
    """ Make sure no extra traits are added.
    """

    x = Int(10)

    traits_view = View(Item("x"), spring)


class MaybeInvalidTrait(HasTraits):

    name = Str()

    name_is_invalid = Property(observe="name")

    traits_view = View(
        Item("name", invalid="name_is_invalid")
    )

    def _get_name_is_invalid(self):
        return len(self.name) < 10


class TestUI(BaseTestMixin, unittest.TestCase):

    def setUp(self):
        BaseTestMixin.setUp(self)

    def tearDown(self):
        BaseTestMixin.tearDown(self)

    @requires_toolkit([ToolkitName.wx])
    def test_reset_with_destroy_wx(self):
        # Characterization test:
        # UI.reset(destroy=True) destroys all ui children of the top control

        foo = FooDialog()
        with create_ui(foo) as ui:

            ui.reset(destroy=True)

            process_cascade_events()
            # the top control is still there
            self.assertIsNotNone(ui.control)
            # but its children are gone
            self.assertEqual(len(ui.control.GetChildren()), 0)

    @requires_toolkit([ToolkitName.qt])
    def test_reset_with_destroy_qt(self):
        # Characterization test:
        # UI.reset(destroy=True) destroys all ui children of the top control

        from pyface import qt

        foo = FooDialog()
        with create_ui(foo) as ui:

            # decorate children's `deleteLater` function to check that it is
            # called on `reset`. check only with the editor parts (only widgets
            # are scheduled.
            # See traitsui.qt4.toolkit.GUIToolkit.destroy_children)
            for c in ui.control.children():
                c.deleteLater = count_calls(c.deleteLater)

            ui.reset(destroy=True)

            # the top control is still there
            self.assertIsNotNone(ui.control)

            # but its children are scheduled for removal
            for c in ui.control.children():
                if isinstance(c, qt.QtGui.QWidget):
                    self.assertEqual(c.deleteLater._n_calls, 1)

    @requires_toolkit([ToolkitName.wx])
    def test_reset_without_destroy_wx(self):
        # Characterization test:
        # UI.reset(destroy=False) destroys all editor controls, but leaves
        # editors and ui children intact

        import wx

        foo = FooDialog()
        with create_ui(foo) as ui:

            self.assertEqual(len(ui._editors), 2)
            self.assertIsInstance(
                ui._editors[0], traitsui.wx.text_editor.SimpleEditor
            )
            self.assertIsInstance(
                ui._editors[0].control, wx.TextCtrl
            )

            ui.reset(destroy=False)

            self.assertEqual(len(ui._editors), 2)
            self.assertIsInstance(
                ui._editors[0], traitsui.wx.text_editor.SimpleEditor
            )
            self.assertIsNone(ui._editors[0].control)

            # children are still there: check first text control
            text_ctrl = ui.control.FindWindowByName("text")
            self.assertIsNotNone(text_ctrl)

    @requires_toolkit([ToolkitName.qt])
    def test_reset_without_destroy_qt(self):
        # Characterization test:
        # UI.reset(destroy=False) destroys all editor controls, but leaves
        # editors and ui children intact

        from pyface import qt

        foo = FooDialog()
        with create_ui(foo) as ui:

            self.assertEqual(len(ui._editors), 2)
            self.assertIsInstance(
                ui._editors[0], traitsui.qt4.text_editor.SimpleEditor
            )
            self.assertIsInstance(ui._editors[0].control, qt.QtGui.QLineEdit)

            ui.reset(destroy=False)

            self.assertEqual(len(ui._editors), 2)
            self.assertIsInstance(
                ui._editors[0], traitsui.qt4.text_editor.SimpleEditor
            )
            self.assertIsNone(ui._editors[0].control)

            # children are still there: check first text control
            text_ctrl = ui.control.findChild(qt.QtGui.QLineEdit)
            self.assertIsNotNone(text_ctrl)

    @requires_toolkit([ToolkitName.wx])
    def test_destroy_after_ok_wx(self):
        # Behavior: after pressing 'OK' in a dialog, the method UI.finish is
        # called and the view control and its children are destroyed

        foo = FooDialog()
        tester = UITester()
        with tester.create_ui(foo) as ui:
            # keep reference to the control to check that it was destroyed
            control = ui.control

            # decorate control's `Destroy` function to check that it is called
            control.Destroy = count_calls(control.Destroy)

            # press the OK button and close the dialog
            ok_button = tester.find_by_id(ui, "OK")
            self.assertEqual(ok_button.inspect(DisplayedText()), "OK")
            self.assertTrue(ok_button.inspect(IsEnabled()))
            ok_button.perform(MouseClick())

            self.assertIsNone(ui.control)
            self.assertEqual(control.Destroy._n_calls, 1)

    @requires_toolkit([ToolkitName.qt])
    def test_destroy_after_ok_qt(self):
        # Behavior: after pressing 'OK' in a dialog, the method UI.finish is
        # called and the view control and its children are destroyed

        foo = FooDialog()
        tester = UITester()
        with tester.create_ui(foo) as ui:
            # keep reference to the control to check that it was deleted
            control = ui.control

            # decorate control's `deleteLater` function to check that it is
            # called
            control.deleteLater = count_calls(control.deleteLater)

            # press the OK button and close the dialog
            ok_button = tester.find_by_id(ui, "OK")
            self.assertEqual(ok_button.inspect(DisplayedText()), "OK")
            self.assertTrue(ok_button.inspect(IsEnabled()))
            ok_button.perform(MouseClick())

            self.assertIsNone(ui.control)
            self.assertEqual(control.deleteLater._n_calls, 1)

    @requires_toolkit([ToolkitName.qt, ToolkitName.wx])
    def test_no_spring_trait(self):
        obj = DisallowNewTraits()
        with create_ui(obj):
            pass

        self.assertTrue("spring" not in obj.traits())

    @requires_toolkit([ToolkitName.qt, ToolkitName.wx])
    def test_invalid_state(self):
        # Regression test for enthought/traitsui#983
        obj = MaybeInvalidTrait(name="Name long enough to be valid")
        with create_ui(obj) as ui:
            editor, = ui.get_editors("name")
            self.assertFalse(editor.invalid)

            obj.name = "too short"
            self.assertTrue(editor.invalid)


# Regression test on an AttributeError commonly seen (enthought/traitsui#1145)
# Code in ui_panel makes use toolkit specific attributes on the toolkit
# specific Editor
ToolkitSpecificEditor = toolkit_object("editor:Editor")


class DummyObject(HasTraits):

    number = Int()


if is_qt():

    from pyface.qt import QtGui, QtCore

    class CustomWidget(QtGui.QWidget):

        def __init__(self, editor, parent=None):
            super().__init__()
            self._some_editor = editor

        def sizeHint(self):
            # This is called if the sibling widget is destroyed (e.g. the
            # nested UI in EditorWithCustomWidget) while the container
            # (e.g. QSplitter in EditorWithCustomWidget) has not been
            # destroyed. The container will want to ask this widget for its
            # sizeHint in order to resize/repaint the layout.
            assert self._some_editor.factory is not None
            return super().sizeHint()

    class EditorWithCustomWidget(ToolkitSpecificEditor):

        def init(self, parent):
            self.control = QtGui.QSplitter(QtCore.Qt.Horizontal)

            widget = CustomWidget(editor=self)
            self.control.addWidget(widget)
            self.control.setStretchFactor(0, 2)

            self._ui = self.edit_traits(
                parent=self.control,
                kind="subpanel",
                view=View(Item("_", label="DUMMY"), width=100, height=100),
            )
            self.control.addWidget(self._ui.control)

        def dispose(self):
            self.dispose_inner_ui()
            super().dispose()

        def dispose_inner_ui(self):
            if self._ui is not None:
                self._ui.dispose()
                self._ui = None

        def update_editor(self):
            pass


if is_wx():

    # The AttributeError is not seen on Wx. But Destroy needs to be made
    # asynchronous.

    import wx
    from traitsui.wx.helper import TraitsUIPanel

    class DummyButtonEditor(ToolkitSpecificEditor):

        def init(self, parent):
            self.control = wx.Button(parent, -1, "Dummy")
            self.control.Bind(wx.EVT_BUTTON, self.update_object)

        def dispose(self):
            # If the object is deleted too soon, we run into a RuntimeError.
            self.control.Unbind(wx.EVT_BUTTON)
            super().dispose()

        def update_object(self, event):
            pass

        def update_editor(self):
            pass

    class EditorWithCustomWidget(ToolkitSpecificEditor):  # noqa: F811

        def init(self, parent):
            self.control = TraitsUIPanel(parent, -1)

            sizer = wx.BoxSizer(wx.HORIZONTAL)
            self._dummy = DummyObject()
            self._ui = self._dummy.edit_traits(
                parent=self.control,
                kind="subpanel",
                view=View(
                    Item(
                        "number",
                        editor=BasicEditorFactory(klass=DummyButtonEditor),
                    ),
                ),
            )
            sizer.Add(self._ui.control, 1, wx.EXPAND)
            self.control.SetSizerAndFit(sizer)

        def update_editor(self):
            pass

        def dispose(self):
            super().dispose()

        def dispose_inner_ui(self):
            if self._ui is not None:
                self._ui.dispose()
                self._ui = None


class TestUIDispose(BaseTestMixin, unittest.TestCase):
    """ Test disposal of UI."""

    def setUp(self):
        BaseTestMixin.setUp(self)

    def tearDown(self):
        BaseTestMixin.tearDown(self)

    @requires_toolkit([ToolkitName.qt, ToolkitName.wx])
    def test_close_ui(self):
        # Test closing the main window normally.
        obj = DummyObject()
        view = View(
            Group(
                Item(
                    "number",
                    editor=BasicEditorFactory(klass=EditorWithCustomWidget),
                ),
            ),
        )
        ui = obj.edit_traits(view=view)
        with ensure_destroyed(ui):
            gui = GUI()
            gui.invoke_later(close_control, ui.control)
            with reraise_exceptions():
                process_cascade_events()
            self.assertIsNone(ui.control)

    @requires_toolkit([ToolkitName.qt, ToolkitName.wx])
    def test_dispose_inner_ui(self):
        obj = DummyObject()
        view = View(
            Group(
                Item(
                    "number",
                    editor=BasicEditorFactory(klass=EditorWithCustomWidget),
                ),
            ),
        )
        ui = obj.edit_traits(view=view)
        editor, = ui.get_editors("number")

        gui = GUI()
        with ensure_destroyed(ui):
            # This represents an event handler that causes a nested UI to be
            # disposed.
            gui.invoke_later(editor.dispose_inner_ui)

            # Allowing GUI to process the disposal requests is crucial.
            # This requirement should be satisfied in production setting
            # where the dispose method is part of an event handler.
            # Not doing so before disposing the main UI would be a programming
            # error in tests settings.
            with reraise_exceptions():
                process_cascade_events()

            gui.invoke_later(close_control, ui.control)
            with reraise_exceptions():
                process_cascade_events()

            self.assertIsNone(ui.control)


@contextlib.contextmanager
def ensure_destroyed(ui):
    """ Ensure the widget is destroyed in the event when test fails."""
    try:
        yield
    finally:
        if ui.control is not None:
            toolkit().destroy_control(ui.control)
        with reraise_exceptions():
            process_cascade_events()


def close_control(control):
    """ Close the widget."""
    if is_qt():
        control.close()
    elif is_wx():
        control.Close()
    else:
        raise NotImplementedError("Unexpected toolkit")



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
**WARNING**

  This demo might not work as expected and some documented features might be
  missing.

-------------------------------------------------------------------------------

This demonstrates one of the simplest cases of using the TraitsUI file dialog
to select a file for opening (i.e. reading or editing).

The first question of course is why use the TraitsUI file dialog at all, when
the standard OS file dialog is also available?

And the answer is that you can use either, but the advantages of using the
TraitsUI file dialog are:

- It supports history. That is, each time the user selects a file for opening,
  the file is added to a persistent history list, similar to many applications
  *Open recent...* function, but built directly into the file dialog.
  The amount of history remembered can be specified by the developer, with the
  default being the last 10 files opened.

- It is resizable. Some standard OS file dialogs are not resizable, which can
  be very annoying to the user trying to select a file through a tiny
  peephole view of the file system. In addition, if the user resizes the
  dialog, the new size and position will be persisted, so that the file dialog
  will appear in the same location the next time the user wants to open a
  file.

- There is a very nice synergy between the file system view and the history
  list. Quite often users shuttle between several *favorite* locations in
  the file system when opening files. The TraitsUI file dialog automatically
  discovers these favorite locations just by the user opening files. When a
  user opens the file dialog, they can select a previously opened file from
  the history list, which then automatically causes the file system view to
  expand the selected file's containing folder, thus allowing them to select a
  different file in the same location. Since the history list is updated each
  time a user selects a file, It tends to automatically discover a *working
  set* of favorite directories just through simple use, without the user
  having to explicitly designate them as such.

- It's customizable. The TraitsUI file dialog accepts extension objects which
  can be used to display additional file information or even modify the
  selection behavior of the dialog. Several extensions are provided with
  TraitsUI (and are demonstrated in some of the other examples), and you are
  free to write your own by implementing a very simple interface.

- The history and user settings are customizable per application. Just by
  setting a unique id in the file dialog request, you can specify that the
  history and window size and position information are specific to your
  application. If you have file dialog extensions added, the user can
  reorder, resize and reconfigure the overall file dialog layout, including
  your extensions, and have their custom settings restored each time they use
  the file dialog. If you do not specify a unique id, then the history and
  user settings default to the system-wide settings for the file dialog. It's
  your choice.

- It's easy to use. That's what this particular example is all about. So take
  a look at the source code for this example to see how easy it is...
"""
# Issue related to the demo warning: enthought/traitsui#953

from traits.api import HasTraits, File, Button

from traitsui.api import View, HGroup, Item

from traitsui.file_dialog import open_file


# -- FileDialogDemo Class -------------------------------------------------
class FileDialogDemo(HasTraits):

    # The name of the selected file:
    file_name = File()

    # The button used to display the file dialog:
    open = Button('Open...')

    # -- Traits View Definitions ----------------------------------------------

    traits_view = View(
        HGroup(
            Item('open', show_label=False),
            '_',
            Item('file_name', style='readonly', springy=True)
        ),
        width=0.5
    )

    # -- Traits Event Handlers ------------------------------------------------

    def _open_changed(self):
        """ Handles the user clicking the 'Open...' button.
        """
        file_name = open_file()
        if file_name != '':
            self.file_name = file_name


# Create the demo:
demo = FileDialogDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Module containing GUI event processing utility for testing purposes.
"""

from traits.etsconfig.api import ETSConfig


def process_cascade_events():
    """ Process all posted events, and attempt to process new events posted by
    the processed events.

    Cautions:
    - An infinite cascade of events will cause this function to enter an
      infinite loop.
    - There still exists technical difficulties with Qt. On Qt4 + OSX,
      QEventLoop.processEvents may report false saying it had found no events
      to process even though it actually had processed some.
      Consequently the internal loop breaks too early such that there are
      still cascaded events unprocessed. Problems are also observed on
      Qt5 + Appveyor occasionally. At the very least, events that are already
      posted prior to calling this function will be processed.
      See enthought/traitsui#951
    """
    if ETSConfig.toolkit.startswith("qt"):
        from pyface.qt import QtCore
        event_loop = QtCore.QEventLoop()
        while event_loop.processEvents(QtCore.QEventLoop.AllEvents):
            pass
    else:
        from pyface.api import GUI
        GUI.process_events()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
This package contains library code for testing TraitsUI applications via
``UITester`` (in ``ui_tester`` module).

Other public modules are also part of the public API.

``_ui_tester_registry`` contains internal implementations to support testing
TraitsUI UI and editors.

The only modules/packages that are specific about TraitsUI UI and editors are
``ui_tester`` and ``_ui_tester_registry``. The other modules are generic enough
to be used with GUI components external to TraitsUI. This allows extensions
beyond the scope of TraitsUI.
"""



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traits.api import *
from traitsui.api import *


class Team(HasTraits):

    batting_order = List(Str)
    roster = List(['Tom', 'Dick', 'Harry', 'Sally'], Str)

    view = View(Item('batting_order', editor=SetEditor(name='roster',
                                                       ordered=True)),
                '_', 'roster@',
                height=500,
                resizable=True)

if __name__ == '__main__':
    Team().configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines an editor for playing animated GIF files.
"""


from wx.adv import Animation, GenericAnimationCtrl

from traits.api import Bool, Str

from traitsui.wx.editor import Editor

from traitsui.basic_editor_factory import BasicEditorFactory


class _AnimatedGIFEditor(Editor):
    """ Editor that displays an animated GIF file.
    """

    # -------------------------------------------------------------------------
    #  Trait definitions:
    # -------------------------------------------------------------------------

    #: Is the animated GIF file currently playing?
    playing = Bool(True)

    def init(self, parent):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self._animate = Animation(self.value)
        self.control = GenericAnimationCtrl(parent, -1, self._animate)
        self.control.SetUseWindowBackgroundColour()
        self.sync_value(self.factory.playing, "playing", "from")
        self.set_tooltip()

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        if not self.playing:
            self.control.Stop()

        self.control.LoadFile(self.value)
        self._file_loaded = True

        if self.playing:
            self.control.Play()

    def _playing_changed(self):
        if self._file_loaded:
            if self.playing:
                self.control.Play()
            else:
                self.control.Stop()


# -------------------------------------------------------------------------
#  Create the editor factory object:
# -------------------------------------------------------------------------

# wxPython editor factory for animated GIF editors:


class AnimatedGIFEditor(BasicEditorFactory):

    #: The editor class to be created:
    klass = _AnimatedGIFEditor

    #: The optional trait used to control whether the animated GIF file is
    #: playing or not:
    playing = Str()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traitsui.qt4.range_editor import (
    LargeRangeSliderEditor,
    LogRangeSliderEditor,
    RangeTextEditor,
    SimpleSliderEditor,
)

from traitsui.testing.tester.command import KeyClick
from traitsui.testing.tester.locator import Slider, Textbox
from traitsui.testing.tester._ui_tester_registry.qt4 import (
    _interaction_helpers,
    _registry_helper
)


class LocatedTextbox:
    """ Wrapper class for a located Textbox in Qt.

    Parameters
    ----------
    textbox : Instance of QtGui.QLineEdit
    """

    def __init__(self, textbox):
        self.textbox = textbox

    @classmethod
    def register(cls, registry):
        """ Class method to register interactions on a LocatedTextbox for the
        given registry.

        If there are any conflicts, an error will occur.

        Parameters
        ----------
        registry : TargetRegistry
            The registry being registered to.
        """
        _registry_helper.register_editable_textbox_handlers(
            registry=registry,
            target_class=cls,
            widget_getter=lambda wrapper: wrapper._target.textbox,
        )


class LocatedSlider:
    """ Wrapper class for a located Slider in Qt.

    Parameters
    ----------
    slider : Instance of QtGui.QSlider
    """

    def __init__(self, slider):
        self.slider = slider

    @classmethod
    def register(cls, registry):
        """ Class method to register interactions on a LocatedSlider for the
        given registry.

        If there are any conflicts, an error will occur.

        Parameters
        ----------
        registry : TargetRegistry
            The registry being registered to.
        """
        registry.register_interaction(
            target_class=cls,
            interaction_class=KeyClick,
            handler=lambda wrapper, interaction:
                _interaction_helpers.key_click_qslider(
                    wrapper._target.slider, interaction, wrapper.delay)
        )


def register(registry):
    """ Register interactions for the given registry.

    If there are any conflicts, an error will occur.

    Parameters
    ----------
    registry : TargetRegistry
        The registry being registered to.
    """

    targets = [SimpleSliderEditor,
               LogRangeSliderEditor,
               LargeRangeSliderEditor]
    for target_class in targets:
        registry.register_location(
            target_class=target_class,
            locator_class=Textbox,
            solver=lambda wrapper, _: LocatedTextbox(
                textbox=wrapper._target.control.text),
        )
        registry.register_location(
            target_class=target_class,
            locator_class=Slider,
            solver=lambda wrapper, _: LocatedSlider(
                slider=wrapper._target.control.slider),
        )
    _registry_helper.register_editable_textbox_handlers(
        registry=registry,
        target_class=RangeTextEditor,
        widget_getter=lambda wrapper: wrapper._target.control,
    )

    LocatedTextbox.register(registry)

    LocatedSlider.register(registry)



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines the various color editors for the Wx user interface toolkit.
"""



import sys

import wx
import wx.adv

from traits.api import List, TraitError
from traitsui.editors.color_editor import (
    ToolkitEditorFactory as BaseToolkitEditorFactory,
)

from .editor_factory import SimpleEditor as BaseSimpleEditor
from .editor_factory import ReadonlyEditor as BaseReadonlyEditor
from .editor_factory import TextEditor as BaseTextEditor

from .color_trait import w3c_color_database
from .helper import TraitsUIPanel


ColorTypes = (wx.Colour,)


# ---------------------------------------------------------------------------
#  The Wx ToolkitEditorFactory class.
# ---------------------------------------------------------------------------


class ToolkitEditorFactory(BaseToolkitEditorFactory):
    """ Wx editor factory for color editors.
    """

    def to_wx_color(self, editor, color=None):
        """ Gets the wxPython color equivalent of the object trait.
        """
        if color is None:
            if self.mapped:
                color = getattr(editor.object, editor.name + "_")
            else:
                color = getattr(editor.object, editor.name)

        if isinstance(color, tuple):
            color = wx.Colour(*[int(round(c * 255.0)) for c in color])
        return color

    def from_wx_color(self, color):
        """ Gets the application equivalent of a wxPython value.
        """
        return color.Red(), color.Green(), color.Blue()

    def str_color(self, color):
        """ Returns the text representation of a specified color value.
        """
        if isinstance(color, ColorTypes):
            alpha = color.Alpha()
            if alpha == 255:
                return "rgb(%d,%d,%d)" % (
                    color.Red(),
                    color.Green(),
                    color.Blue(),
                )

            return "rgba(%d,%d,%d,%d)" % (
                color.Red(),
                color.Green(),
                color.Blue(),
                alpha,
            )

        return str(color)


# ------------------------------------------------------------------------------
#  'ColorComboBox' class:
# ------------------------------------------------------------------------------


class ColorComboBox(wx.adv.OwnerDrawnComboBox):

    def OnDrawItem(self, dc, rect, item, flags):

        r = wx.Rect(rect.x, rect.y, rect.width, rect.height)
        r.Deflate(3, 0)
        swatch_size = r.height - 2

        color_name = self.GetString(item)

        dc.DrawText(
            color_name, r.x + 3, r.y + (r.height - dc.GetCharHeight()) // 2
        )

        if color_name == "custom":
            swatch = wx.Rect(
                r.x + r.width - swatch_size, r.y + 1, swatch_size, swatch_size
            )
            dc.GradientFillLinear(
                swatch, wx.Colour(255, 255, 0), wx.Colour(0, 0, 255)
            )
        else:
            color = w3c_color_database.Find(color_name)

            brush = wx.Brush(color)
            dc.SetBrush(brush)
            dc.DrawRectangle(
                r.x + r.width - swatch_size, r.y + 1, swatch_size, swatch_size
            )


# ------------------------------------------------------------------------------
#  'SimpleColorEditor' class:
# ------------------------------------------------------------------------------


class SimpleColorEditor(BaseSimpleEditor):
    """ Simple style of color editor, which displays a text field whose
    background color is the color value. Selecting the text field displays
    a dialog box for selecting a new color value.
    """

    # --------------------------------------------------------------------------
    #  Invokes the pop-up editor for an object trait:
    # --------------------------------------------------------------------------

    choices = List()

    def _choices_default(self):
        """ by default, uses the W3C 16 color names.
        """
        return [
            "aqua",
            "black",
            "blue",
            "fuchsia",
            "gray",
            "green",
            "lime",
            "maroon",
            "navy",
            "olive",
            "purple",
            "red",
            "silver",
            "teal",
            "white",
            "yellow",
            "custom",
        ]

    def init(self, parent):
        """
        Finishes initializing the editor by creating the underlying widget.
        """
        current_color = self.factory.to_wx_color(self)
        current_color_name = current_color.GetAsString()

        self.control = ColorComboBox(
            parent,
            -1,
            current_color_name,
            wx.Point(0, 0),
            wx.Size(40, -1),
            self.choices,
            style=wx.CB_READONLY,
        )

        self.control.Bind(wx.EVT_COMBOBOX, self.color_selected)

    def dispose(self):
        if self.control is not None:
            self.control.Unbind(wx.EVT_COMBOBOX, handler=self.color_selected)
        super().dispose()

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        current_color = self.factory.to_wx_color(self)
        self.control.SetValue(self.string_value(current_color))

    def color_selected(self, event):
        """
        Event for when color is selected
        """

        color_name = self.choices[event.Selection]

        if color_name == "custom":
            color_dialog = wx.ColourDialog(self.control)
            result = color_dialog.ShowModal()
            if result == wx.ID_CANCEL:
                return

            color = color_dialog.GetColourData().GetColour()
            self.value = self.factory.from_wx_color(color)
        else:
            try:
                color = w3c_color_database.Find(color_name)
                self.value = self.factory.from_wx_color(color)
            except ValueError:
                pass

        return

    def string_value(self, color):
        """ Returns the text representation of a specified color value.
        """
        color_name = w3c_color_database.FindName(color)
        if color_name != "":
            return color_name

        return color.GetAsString()


# ------------------------------------------------------------------------------
#  'CustomColorEditor' class:
# ------------------------------------------------------------------------------


class CustomColorEditor(BaseSimpleEditor):
    """ Simple style of color editor, which displays a text field whose
    background color is the color value. Selecting the text field displays
    a dialog box for selecting a new color value.
    """

    def init(self, parent):
        """
        Finishes initializing the editor by creating the underlying widget.
        """
        self.control = self._panel = parent = TraitsUIPanel(parent, -1)
        sizer = wx.BoxSizer(wx.HORIZONTAL)

        # 'text_control' is the text display of the color.
        text_control = wx.TextCtrl(
            parent, -1, self.str_value, style=wx.TE_PROCESS_ENTER
        )
        text_control.Bind(wx.EVT_KILL_FOCUS, self.update_object)
        parent.Bind(wx.EVT_TEXT_ENTER, self.update_object, id=text_control.GetId())

        # 'button_control' shows the 'Edit' button.
        button_control = wx.Button(parent, label="Edit", style=wx.BU_EXACTFIT)
        button_control.Bind(wx.EVT_BUTTON, self.open_color_dialog, id=button_control.GetId())

        sizer.Add(text_control, wx.ALIGN_LEFT)
        sizer.AddSpacer(8)
        sizer.Add(button_control, wx.ALIGN_RIGHT)
        self.control.SetSizer(sizer)

        self._text_control = text_control
        self._button_control = button_control

        self.set_tooltip()

        return

    def update_object(self, event):
        """ Handles the user changing the contents of the edit control.
        """
        if not isinstance(event, wx._core.CommandEvent):
            event.Skip()
            return
        try:
            # The TextCtrl object was saved as self._text_control in init().
            value = self._text_control.GetValue()
            self.value = w3c_color_database.Find(value)
            self.set_color()
        except TraitError:
            pass

    def set_color(self):
        # The CustomColorEditor uses this method instead of the global
        # set_color function.
        color = self.factory.to_wx_color(self)
        self._text_control.SetBackgroundColour(color)
        self.control.SetBackgroundColour(color)
        self._text_control.SetValue(self.string_value(color))

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        self.set_color()

    def open_color_dialog(self, event):
        """ Opens the color dialog and sets the value upon return
        """
        color_data = wx.ColourData()
        color_data.SetColour(self.value)

        color_dialog = wx.ColourDialog(self.control, color_data)
        result = color_dialog.ShowModal()
        if result == wx.ID_CANCEL:
            return

        color = color_dialog.GetColourData().GetColour()
        self.value = color
        self.set_color()

    def color_selected(self, event):
        """
        Event for when color is selected
        """
        color = event.GetColour()
        try:
            self.value = self.factory.from_wx_color(color)
        except ValueError:
            pass

        return

    def string_value(self, color):
        """ Returns the text representation of a specified color value.
        """
        color_name = w3c_color_database.FindName(color)
        if color_name != "":
            return color_name

        return self.factory.str_color(color)


# ------------------------------------------------------------------------------
#  'ReadonlyColorEditor' class:
# ------------------------------------------------------------------------------


class ReadonlyColorEditor(BaseReadonlyEditor):
    """ Read-only style of color editor, which displays a read-only text field
    whose background color is the color value.
    """

    def init(self, parent):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        self.control = wx.TextCtrl(parent, style=wx.TE_READONLY)

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        # super().update_editor()
        self.control.SetValue(self.string_value(self.value))
        set_color(self)

    def string_value(self, color):
        """ Returns the text representation of a specified color value.
        """
        color_name = w3c_color_database.FindName(color)
        if color_name != "":
            return color_name

        return self.factory.str_color(color)


# ------------------------------------------------------------------------------
#  'ReadonlyColorEditor' class:
# ------------------------------------------------------------------------------


class TextColorEditor(BaseTextEditor):
    """ Text style of color editor, which displays a text field
    whose background color is the color value.
    """

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        self.control.SetValue(self.string_value(self.value))
        set_color(self)

    def update_object(self, event):
        """ Handles the user changing the contents of the edit control.
        """
        if not isinstance(event, wx._core.CommandEvent):
            return
        try:
            self.value = w3c_color_database.Find(self.control.GetValue())
            set_color(self)
        except TraitError:
            pass

    def string_value(self, color):
        """ Returns the text representation of a specified color value.
        """
        color_name = w3c_color_database.FindName(color)
        if color_name != "":
            return color_name

        return self.factory.str_color(color)


# ------------------------------------------------------------------------------
#   Sets the color of the specified editor's color control:
# ------------------------------------------------------------------------------


def set_color(editor):
    """  Sets the color of the specified color control.
    """
    color = editor.factory.to_wx_color(editor)
    editor.control.SetBackgroundColour(color)


# Define the SimpleEditor, CustomEditor, etc. classes which are used by the
# editor factory for the color editor.
SimpleEditor = SimpleColorEditor
CustomEditor = CustomColorEditor
TextEditor = TextColorEditor
ReadonlyEditor = ReadonlyColorEditor



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines the wxPython implementation of the editor used to wrap a non-Traits
based custom control.
"""


import wx

from .editor import Editor

# -------------------------------------------------------------------------
#  'CustomEditor' class:
# -------------------------------------------------------------------------


class CustomEditor(Editor):
    """ Wrapper for a custom editor control
    """

    def init(self, parent):
        """ Finishes initializing the editor by creating the underlying toolkit
            widget.
        """
        factory = self.factory.factory
        if factory is not None:
            self.control = factory(*((parent, self) + self.factory.args))
        if self.control is None:
            self.control = control = wx.StaticText(
                parent,
                -1,
                "An error occurred creating a custom editor.\n"
                "Please contact the developer.",
            )
            control.SetBackgroundColour(wx.RED)
            control.SetForegroundColour(wx.WHITE)
        self.set_tooltip()

    def update_editor(self):
        """ Updates the editor when the object trait changes externally to the
            editor.
        """
        pass



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

from traitsui.qt4.ui_base import ButtonEditor
from traitsui.testing.tester.command import MouseClick
from traitsui.testing.tester.query import DisplayedText
from traitsui.testing.tester._ui_tester_registry.qt4 import (
    _interaction_helpers
)


def register(registry):
    """ Register solvers/handlers specific to qt ui_base Editors
    for the given registry.

    If there are any conflicts, an error will occur.

    Parameters
    ----------
    registry : TargetRegistry
    """
    handlers = [
        (MouseClick,
            (lambda wrapper, _: _interaction_helpers.mouse_click_qwidget(
                wrapper._target.control, wrapper.delay))),
        (DisplayedText,
            lambda wrapper, _: wrapper._target.control.text()),
    ]
    for interaction_class, handler in handlers:
        registry.register_interaction(
            target_class=ButtonEditor,
            interaction_class=interaction_class,
            handler=handler
        )



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
A timer editor

Display or edit a time.

You can edit the time directly, or by using only the arrow keys (left & right
to navigate, up & down to change).

"""

import datetime

from traits.api import HasTraits, Time
from traitsui.api import View, Item, TimeEditor


class TimeEditorDemo(HasTraits):
    """ Demo class. """
    time = Time(datetime.time(12, 0, 0))

    traits_view = View(
        Item('time', label='Simple Editor'),
        Item(
            'time',
            label='Readonly Editor',
            style='readonly',
            # Show 24-hour mode instead of default 12 hour.
            editor=TimeEditor(strftime='%H:%M:%S')
        ),
        resizable=True
    )

    def _time_changed(self):
        """ Print each time the time value is changed in the editor. """
        print(self.time)


# -- Set Up The Demo ------------------------------------------------------

demo = TimeEditorDemo()

if __name__ == "__main__":
    demo.configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
This demo shows the proper way to create a **Property** whose value is a
list, especially when the value of the **Property** will be used in a user
interface, such as with the **TableEditor**.

Most of the demo is just the machinery to set up the example. The key thing to
note is the declaration of the *people* trait:

    people = Property( List, observe = 'ticker' )

In this case, by defining the **Property** as having a value of type
**List**, you are ensuring that the computed value of the property will be
validated using the **List** type, which in addition to verifying that the
value is indeed a list, also guarantees that it will be converted to a
**TraitListObject**, which is necessary for correct interaction with various
Traits UI editors in a user interface.

Note also the use of the *observe* metadata to trigger a trait property
change whenever the *ticker* trait changes (in this case, it is changed
every three seconds by a background thread).

Finally, the use of the *@cached_property* decorator simplifies the
implementation of the property by allowing the **_get_people** *getter*
method to perform the expensive generation of a new list of people only when
the *ticker* event fires, not every time it is accessed.
"""

from random import randint, choice

from threading import Thread

from time import sleep

from traits.api import (
    HasStrictTraits, HasPrivateTraits, Str, Int, Enum, List, Event, Property,
    cached_property
)

from traitsui.api import ObjectColumn, Item, TableEditor, View


# -- Person Class ---------------------------------------------------------
class Person(HasStrictTraits):
    """ Defines some sample data to display in the TableEditor.
    """

    name = Str()
    age = Int()
    gender = Enum('Male', 'Female')


# -- PropertyListDemo Class -----------------------------------------------
class PropertyListDemo(HasPrivateTraits):
    """ Displays a random list of Person objects in a TableEditor that is
        refreshed every 3 seconds by a background thread.
     """

    # An event used to trigger a Property value update:
    ticker = Event()

    # The property being display in the TableEditor:
    people = Property(List, observe='ticker')

    # Tiny hack to allow starting the background thread easily:
    begin = Int()

    # -- Traits View Definitions ----------------------------------------------

    traits_view = View(
        Item(
            'people',
            show_label=False,
            editor=TableEditor(
                columns=[
                    ObjectColumn(name='name', editable=False, width=0.50),
                    ObjectColumn(name='age', editable=False, width=0.15),
                    ObjectColumn(name='gender', editable=False, width=0.35)
                ],
                auto_size=False,
                show_toolbar=False,
                sortable=False,
             )
         ),
        title='Property List Demo',
        width=0.25,
        height=0.33,
        resizable=True
    )

    # -- Property Implementations ---------------------------------------------
    @cached_property
    def _get_people(self):
        """ Returns the value for the 'people' property.
        """
        return [
            Person(
                name='%s %s' % (
                    choice(['Tom', 'Dick', 'Harry', 'Alice', 'Lia', 'Vibha']),
                    choice(['Thomas', 'Jones', 'Smith', 'Adams', 'Johnson'])
                ),
                age=randint(21, 75),
                gender=choice(['Male', 'Female'])
            )
            for i in range(randint(10, 20))
        ]

    # -- Default Value Implementations ----------------------------------------
    def _begin_default(self):
        """ Starts the background thread running.
        """
        thread = Thread(target=self._timer)
        thread.daemon = True
        thread.start()

        return 0

    # -- Private Methods ------------------------------------------------------

    def _timer(self):
        """ Triggers a property update every 3 seconds for 30 seconds.
        """
        for i in range(10):
            sleep(3)
            self.ticker = True


# Create the demo:
demo = PropertyListDemo()
demo.begin

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines the UIInfo class used to represent the object and editor content of
    an active Traits-based user interface.
"""

from traits.api import HasPrivateTraits, Instance, Constant, Bool


class UIInfo(HasPrivateTraits):
    """ Represents the object and editor content of an active Traits-based
    user interface
    """

    # -------------------------------------------------------------------------
    #  Trait definitions:
    # -------------------------------------------------------------------------

    #: Bound to a UI object at UIInfo construction time
    ui = Instance("traitsui.ui.UI", allow_none=True)

    #: Indicates whether the UI has finished initialization
    initialized = Bool(False)

    def bind_context(self):
        """ Binds all of the associated context objects as traits of the
            object.
        """
        for name, value in self.ui.context.items():
            self.bind(name, value)

    def bind(self, name, value, id=None):
        """ Binds a name to a value if it is not already bound.
        """
        if id is None:
            id = name

        if not hasattr(self, name):
            self.add_trait(name, Constant(value))
            if id != "":
                self.ui._names.append(id)



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines the editor factory used to wrap a non-Traits based custom control.
"""

from traits.api import Callable, Tuple, Property

from traitsui.basic_editor_factory import BasicEditorFactory
from traitsui.toolkit import toolkit_object


class CustomEditor(BasicEditorFactory):
    """ Editor factory for custom editors.
    """

    #: Editor class to be instantiated.
    klass = Property()

    #: Factory function used to create the custom control
    factory = Callable()

    #: Arguments to be passed to the user's custom editor factory
    args = Tuple()

    def __init__(self, *args, **traits):
        if len(args) >= 1:
            self.factory = args[0]
            self.args = args[1:]
        super().__init__(**traits)

    # -------------------------------------------------------------------------
    #  Property getters
    # -------------------------------------------------------------------------
    def _get_klass(self):
        """ Returns the editor class to be created.
        """
        return toolkit_object("custom_editor:CustomEditor")


# This alias is deprecated and will be removed in TraitsUI 8.
ToolkitEditorFactory = CustomEditor



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

"""
**WARNING**

  This demo might not work as expected and some documented features might be
  missing.

-------------------------------------------------------------------------------

This demonstrates using the TraitsUI file dialog with a file dialog extension,
in this case, the **FileInfo** extension, which displays information about
the currently selected file, such as:

- File size.
- Date and time last accessed.
- Date and time last modified.
- Date and time last created.

For more information about why you would want to use the TraitsUI file dialog
over the standard OS file dialog, select the **File Open** demo. For a
demonstration of writing a custom file dialog extension, select the
**File Open with Custom Extension** demo.
"""
# Issue related to the demo warning: enthought/traitsui#953

from traits.api import HasTraits, File, Button

from traitsui.api import View, HGroup, Item

from traitsui.file_dialog import open_file, FileInfo


# -- FileDialogDemo Class -------------------------------------------------

# Demo specific file dialig id:
demo_id = 'traitsui.demo.standard_editors.file_dialog.file_info'


class FileDialogDemo(HasTraits):

    # The name of the selected file:
    file_name = File()

    # The button used to display the file dialog:
    open = Button('Open...')

    # -- Traits View Definitions ----------------------------------------------

    traits_view = View(
        HGroup(
            Item('open', show_label=False),
            '_',
            Item('file_name', style='readonly', springy=True)
        ),
        width=0.5
    )

    # -- Traits Event Handlers ------------------------------------------------

    def _open_changed(self):
        """ Handles the user clicking the 'Open...' button.
        """
        file_name = open_file(extensions=FileInfo(), id=demo_id)
        if file_name != '':
            self.file_name = file_name


# Create the demo:
demo = FileDialogDemo()

# Run the demo (if invoked from the command line):
if __name__ == '__main__':
    demo.configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

# key_bindings.py -- Example of a code editor with a key bindings editor

from traits.api import Button, Code, HasPrivateTraits, Str
from traitsui.api import Group, Handler, Item, View
from traitsui.key_bindings import KeyBinding, KeyBindings

key_bindings = KeyBindings(
    KeyBinding(
        binding1='Ctrl-s',
        description='Save to a file',
        method_name='save_file',
    ),
    KeyBinding(
        binding1='Ctrl-r',
        description='Run script',
        method_name='run_script',
    ),
    KeyBinding(
        binding1='Ctrl-k',
        description='Edit key bindings',
        method_name='edit_bindings',
    ),
)


class CodeHandler(Handler):
    """ Handler class for bound methods. """

    def save_file(self, info):
        info.object.status = "save file"

    def run_script(self, info):
        info.object.status = "run script"

    def edit_bindings(self, info):
        info.object.status = "edit bindings"
        key_bindings.edit_traits()


class KBCodeExample(HasPrivateTraits):

    code = Code()
    status = Str()
    kb = Button(label='Edit Key Bindings')

    view = View(
        Group(
            Item('code', style='custom', resizable=True),
            Item('status', style='readonly'),
            'kb',
            orientation='vertical',
            show_labels=False,
        ),
        id='KBCodeExample',
        key_bindings=key_bindings,
        title='Code Editor With Key Bindings',
        resizable=True,
        handler=CodeHandler(),
    )

    def _kb_fired(self, event):
        key_bindings.edit_traits()


if __name__ == '__main__':
    KBCodeExample().configure_traits()



# (C) Copyright 2004-2021 Enthought, Inc., Austin, TX
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in LICENSE.txt and may be redistributed only under
# the conditions described in the aforementioned license. The license
# is also available online at http://www.enthought.com/licenses/BSD.txt
#
# Thanks for using Enthought open source!

""" Defines the button editor factory for all traits toolkit backends.
"""

from pyface.ui_traits import Image
from traits.api import Str, Range, Enum, Property, Either

from traitsui.editor_factory import EditorFactory
from traitsui.ui_traits import AView
from traitsui.view import View


class ButtonEditor(EditorFactory):
    """ Editor factory for buttons.
    """

    # -------------------------------------------------------------------------
    #  Trait definitions:
    # -------------------------------------------------------------------------

    #: Value to set when the button is clicked
    value = Property()

    #: Optional label for the button
    label = Str()

    #: The name of the external object trait that the button label is synced to
    label_value = Str()

    #: The name of the trait on the object that contains the list of possible
    #: values.  If this is set, then the value, label, and label_value traits
    #: are ignored; instead, they will be set from this list.  When this button
    #: is clicked, the value set will be the one selected from the drop-down.
    values_trait = Either(None, Str)

    #: (Optional) Image to display on the button
    image = Image

    #: The name of the external object trait that the button image is synced to
    image_value = Str()

    #: Extra padding to add to both the left and the right sides

    width_padding = Range(0, 31, 7)

    #: Extra padding to add to both the top and the bottom sides
    height_padding = Range(0, 31, 5)

    #: Presentation style
    style = Enum("button", "radio", "toolbar", "checkbox")

    #: Orientation of the text relative to the image
    orientation = Enum("vertical", "horizontal")

    #: The optional view to display when the button is clicked:
    view = AView

    # -------------------------------------------------------------------------
    #  Traits view definition:
    # -------------------------------------------------------------------------

    traits_view = View(["label", "value", "|[]"])

    def _get_value(self):
        return self._value

    def _set_value(self, value):
        self._value = value
        if isinstance(value, str):
            try:
                self._value = int(value)
            except ValueError:
                try:
                    self._value = float(value)
                except ValueError:
                    pass

    def __init__(self, **traits):
        self._value = 0
        super().__init__(**traits)


# This alias is deprecated and will be removed in TraitsUI 8.
ToolkitEditorFactory = ButtonEditor
